@@ -47,7 +47,6 @@ class PostgreSQLQueryProcessor(object):
             self._query = self._strip_query(query)
             self.process_query()
 
-
     def _extract_instances(self, column_keyword_function_listener):
         select_list_columns = []
         other_columns = []
@@ -86,7 +85,8 @@ class PostgreSQLQueryProcessor(object):
 
             if isinstance(i[1], PostgreSQLParser.Select_listContext):
                 if len(i) == 3:
-                    select_list_columns.append(i[2])
+                    select_list_columns.append([[i[2][0][0] + [i[1]],
+                                                i[2][0][1]]])
                     ctx_stack.append(i)
 
             if isinstance(i[1], PostgreSQLParser.Where_clauseContext) or\
@@ -113,7 +113,8 @@ class PostgreSQLQueryProcessor(object):
                             if ju[0][1] is None:
                                 other_columns.append([[[ctx[2][0][0],
                                                         ctx[2][0][1],
-                                                        ju[0][2], ctx], None]])
+                                                        ju[0][2],
+                                                        ctx[1]], None]])
                 elif i[1].ON():
                     if len(i[2]) > 1:
                         for j in i[2]:
@@ -143,6 +144,7 @@ class PostgreSQLQueryProcessor(object):
 
     def _get_budget_column(self, c, tab, ref):
         cname = c[0][2]
+        cctx = c[0][3]
         calias = c[1]
         t = tab
 
@@ -165,7 +167,7 @@ class PostgreSQLQueryProcessor(object):
                 column_found = True
                 break
 
-        return cname, calias, column_found, t
+        return cname, cctx, calias, column_found, t
 
     def _extract_columns(self, columns, select_list_tables, ref_dict, join,
                          budget, column_aliases, touched_columns=None,
@@ -181,6 +183,7 @@ class PostgreSQLQueryProcessor(object):
             c = col[0]
 
             cname = c[0][2]
+            cctx = c[0][3]
             calias = c[1]
 
             # if * is selected we don't care too much
@@ -207,7 +210,7 @@ class PostgreSQLQueryProcessor(object):
                 if join and c[0][2] != '*' and\
                         (tab[1] != c[0][1] or
                          (tab[1] is None and c[0][1] is None)):
-                    cname, calias, column_found, tab =\
+                    cname, cctx, calias, column_found, tab =\
                             self._get_budget_column(c, tab, budget[-1][2])
                     # raise an ambigous column
                     if column_found and c[0][1] is None:
@@ -224,7 +227,7 @@ class PostgreSQLQueryProcessor(object):
 
                 if isinstance(ref[0], int):
                     # ref is a budget column
-                    cname, calias, column_found, tab =\
+                    cname, cctx, calias, column_found, tab =\
                             self._get_budget_column(c, tab, ref[2])
 
                     ref_cols = [j[0][2] for j in ref[2]]
@@ -240,6 +243,7 @@ class PostgreSQLQueryProcessor(object):
             except KeyError:
                 if None not in c[0]:
                     cname = c[0][2]
+                    cctx = c[0][3]
                     calias = c[1]
                     tab = [[c[0][0], c[0][1]]]
                     column_found = True
@@ -249,7 +253,7 @@ class PostgreSQLQueryProcessor(object):
                     if subquery_contents is not None:
                         try:
                             contents = subquery_contents[c[0][1]]
-                            cname, calias, column_found, tab =\
+                            cname, cctx, calias, column_found, tab =\
                                 self._get_budget_column(c, tab, contents)
 
                         except KeyError:
@@ -260,6 +264,7 @@ class PostgreSQLQueryProcessor(object):
                             for t in tabs:
                                 if t[1] == c[0][1]:
                                     cname = c[0][2]
+                                    cctx = c[0][3]
                                     calias = c[1]
                                     tab = [t]
                                     column_found = True
@@ -280,10 +285,12 @@ class PostgreSQLQueryProcessor(object):
                         else:
 
                             missing_columns.append(c)
-                            columns[i] = [[c[0][0], c[0][1], c[0][2], c[0][3]], c[1]]
+                            columns[i] = [[c[0][0], c[0][1],
+                                           c[0][2], c[0][3]], c[1]]
                             if touched_columns is not None:
                                 touched_columns.append([[c[0][0], c[0][1],
-                                                       c[0][2], c[0][3]], c[1]])
+                                                       c[0][2], c[0][3]],
+                                                       c[1]])
                         continue
 
                 elif c[0][2] is not None and c[0][2] != '*' and c[0][1] is \
@@ -295,7 +302,7 @@ class PostgreSQLQueryProcessor(object):
                     column_found = False
 
                     if isinstance(ref[0], int):
-                        cname, calias, column_found, tab =\
+                        cname, cctx, calias, column_found, tab =\
                                 self._get_budget_column(c, tab, ref[2])
 
                         # We allow None.None colunms because they are produced
@@ -305,14 +312,14 @@ class PostgreSQLQueryProcessor(object):
                             raise QueryError("Unknown column '%s'." % c[0][2])
 
             if touched_columns is not None:
-                touched_columns.append([[tab[0][0], tab[0][1], cname], calias])
+                touched_columns.append([[tab[0][0], tab[0][1], cname, cctx],
+                                        calias])
             else:
                 columns[i] = [[tab[0][0], tab[0][1], cname, c[0][3]], calias]
 
         for i in remove_column_idxs[::-1]:
             columns.pop(i)
 
-        columns.extend(extra_columns)
         return missing_columns
 
     def process_query(self, replace_schema_name={}):
@@ -433,7 +440,6 @@ class PostgreSQLQueryProcessor(object):
                                        ref_dict, join, budget,
                                        column_aliases_from_previous,
                                        touched_columns)
-
             missing_columns.extend([[i] for i in mc])
 
             if join:
@@ -459,8 +465,7 @@ class PostgreSQLQueryProcessor(object):
                                        column_aliases_from_previous,
                                        touched_columns, subquery_contents)
             if len(mc):
-                #  print(mc)
-                unref_cols = "', '".join(['.'.join([j for j in i[0] if j])
+                unref_cols = "', '".join(['.'.join([j for j in i[0][:3] if j])
                                          for i in mc])
                 raise QueryError("Unreferenced column(s): '%s'." % unref_cols)
 
@@ -491,26 +496,24 @@ class PostgreSQLQueryProcessor(object):
                         acol[2] != col[2]:
                     del_columns.append(col)
 
-        self.columns = [(i[0].lstrip('"').rstrip('"'),
-                         i[1].lstrip('"').rstrip('"'),
-                         i[2].lstrip('"').rstrip('"'))
-                        for i in
-                        list(set(touched_columns).difference(del_columns))]
+        columns = list(set(touched_columns).difference(del_columns))
+        self.columns = list(set([self._strip_column(i) for i in columns]))
         self.keywords = list(set(keywords))
         self.functions = list(set(functions))
         self.display_columns = [(i[0].lstrip('"').rstrip('"'),
-                                [i[1][0].lstrip('"').rstrip('"'),
-                                 i[1][1].lstrip('"').rstrip('"'),
-                                 i[1][2].lstrip('"').rstrip('"')])
+                                list(self._strip_column(i[1])))
                                 for i in display_columns]
-        self.tables = [[i[0].lstrip('"').rstrip('"'),
-                        i[1].lstrip('"').rstrip('"')] for i in tables]
 
-        # If there are any pg_sphere object that are indexed we need
-        # to replace the ADQL translated query parts with the column
+        self.tables = list(set([tuple([i[0].lstrip('"').rstrip('"')
+                        if i[0] is not None else i[0],
+                        i[1].lstrip('"').rstrip('"')
+                        if i[1] is not None else i[1]]) for i in tables]))
+
+        # If there are any pg_sphere objects that are indexed we need
+        # to replace the ADQL translated query parts with the indexed column
         # names
         if self._indexed_objects is not None:
-            pg_sphere_listener = PgSphereListener(ref_dict,
+            pg_sphere_listener = PgSphereListener(columns,
                                                   self._indexed_objects)
             self.walker.walk(pg_sphere_listener, tree)
             for k, v in pg_sphere_listener.replace_dict.items():
@@ -527,6 +530,13 @@ class PostgreSQLQueryProcessor(object):
     def _strip_query(self, query):
         return query.lstrip('\n').rstrip().rstrip(';') + ';'
 
+    def _strip_column(self, col):
+        scol = [None, None, None]
+        for i in range(3):
+            if col[i] is not None:
+                scol[i] = col[i].lstrip('"').rstrip('"')
+        return tuple(scol)
+
     def set_query(self, query):
         """
         Helper to set the query string.
